﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.Devices.Sensors;
using Windows.UI.Input;


// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=391641

namespace SensorUnlock
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        Accelerometer accelSensor;        
        Point3D accelReading = new Point3D();

        //Inclinometer myIncMeter = null;
        //float inclX = 0;
        //float inclY = 0;

        GestureRecognizer gestureRecognizer = new GestureRecognizer();
        Windows.UI.Xaml.UIElement element;  
        List<uint> pointerList = new List<uint>();       

        LastMove lastMove = new LastMove();
        double lastMagnitude = 0;
        const double lastMoveThreshold = 0.9;

        private Point initialpoint;      

        int lastTouchPoints = 0;

        public MainPage()
        {
            this.InitializeComponent();
            this.NavigationCacheMode = NavigationCacheMode.Required;
            GestureInputProcessor(gestureRecognizer, rectGestureArea);
        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.
        /// This parameter is typically used to configure the page.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            accelSensor = Accelerometer.GetDefault();
            accelSensor.ReportInterval = 300;
            //myIncMeter = Inclinometer.GetDefault();
            
            if (accelSensor != null)
            {
                accelSensor.ReadingChanged += AccelerometerReadingChanged;
            }

            // TODO: If your application contains multiple pages, ensure that you are
            // handling the hardware Back button by registering for the
            // Windows.Phone.UI.Input.HardwareButtons.BackPressed event.
            // If you are using the NavigationHelper provided by some templates,
            // this event is handled for you.
        }
        public async void AccelerometerReadingChanged(object sender, AccelerometerReadingChangedEventArgs e)
        {
            try
            {
                await this.Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () =>
                    {
                        //InclinometerReading incRead = myIncMeter.GetCurrentReading();
                        //inclX = incRead.RollDegrees;
                        //inclY = incRead.PitchDegrees;

                        accelReading.X = (float)e.Reading.AccelerationX;// -((Math.Cos(inclY * Math.PI / 180) * Math.Sin(inclX * Math.PI / 180)));
                        accelReading.Y = (float)e.Reading.AccelerationY;// -(-Math.Sin(inclY * Math.PI / 180));
                        accelReading.Z = (float)e.Reading.AccelerationZ;// -(-(Math.Cos(inclX * Math.PI / 180) * Math.Cos(inclY * Math.PI / 180)));

                        if ((Math.Sqrt((accelReading.X * accelReading.X) + (accelReading.Y * accelReading.Y))) > lastMoveThreshold)
                        {
                            lastMove = calculateMove(new DateTime(), accelReading.X, accelReading.Y);
                        }
                        if (DateTime.Compare(DateTime.Now, lastMove.Time) > 1600)
                        {
                            lastMove.Time = DateTime.Now;
                            lastMove.Magnitute = 0;
                        }
                        

                        txtAccelX.Text = "X: " + accelReading.X;
                        txtAccelY.Text = "Y: " + accelReading.Y;
                        txtAccelZ.Text = "Z: " + accelReading.Z;
                    });
            }
            catch (Exception ex)
            {
                string errorMessage = ex.Message;
            }
        }

        private void Canvas_PointerPressed(object sender, PointerRoutedEventArgs e)
        {
            var ps = e.GetIntermediatePoints(null);
            if (ps != null && ps.Count > 0)
            {
                //bool _hasCapture = ((Border)sender).CapturePointer(e.Pointer);
                this.gestureRecognizer.ProcessDownEvent(ps[0]);
                this.pointerList.Add(e.Pointer.PointerId);

                lastTouchPoints = pointerList.Count;
                txtNumTouches.Text = "Touch Points: " + pointerList.Count + " /// NA";
                e.Handled = true;
            }
        }

        private void gestureRecognizer_ManipulationStarted(object sender, ManipulationStartedRoutedEventArgs e)
        {
            initialpoint = e.Position;
        }

        private void gestureRecognizer_ManipulationDelta(object sender, ManipulationDeltaRoutedEventArgs e)
        {
            if (e.IsInertial)
            {
                if (lastMove.Magnitute > 0)
                {
                    System.Diagnostics.Debug.WriteLine("Magnitude: " + lastMove.Magnitute);
                    txtStatus.Text = "Magnitude: " + lastMove.Magnitute;
                }

                Point currentpoint = e.Position;
                System.Diagnostics.Debug.WriteLine("Initial Point: " + initialpoint.X + " " + initialpoint.Y);
                System.Diagnostics.Debug.WriteLine("Ending Point: " + e.Position.X + " " + e.Position.Y);
                if (currentpoint.X - initialpoint.X >= 300)//500 is the threshold value, where you want to trigger the swipe right event
                {
                    System.Diagnostics.Debug.WriteLine("Swipe Right");
                    if (lastMove.Magnitute > lastMoveThreshold)
                    {
                        System.Diagnostics.Debug.WriteLine("Magnitude: " + lastMove.Magnitute);
                        txtStatus.Text = "Unlocked with " + lastTouchPoints + " fingers RIGHT";
                    }
                    e.Complete();
                }
                else if (currentpoint.X - initialpoint.X <= -300)
                {
                    System.Diagnostics.Debug.WriteLine("Swipe Left");
                    if (lastMove.Magnitute > lastMoveThreshold)
                    {
                        System.Diagnostics.Debug.WriteLine("Magnitude: " + lastMove.Magnitute);
                        txtStatus.Text = "Unlocked with " + lastTouchPoints + " fingers LEFT";
                    }
                    e.Complete();
                }
                else if (currentpoint.Y - initialpoint.Y >= 300)
                {
                    System.Diagnostics.Debug.WriteLine("Swipe Down");
                    if (lastMove.Magnitute > lastMoveThreshold)
                    {
                        System.Diagnostics.Debug.WriteLine("Magnitude: " + lastMove.Magnitute);
                        txtStatus.Text = "Unlocked with " + lastTouchPoints + " fingers DOWN";
                    }
                    e.Complete();
                }
                else if (currentpoint.Y - initialpoint.Y <= -300)
                {
                    System.Diagnostics.Debug.WriteLine("Swipe Up");
                    if (lastMove.Magnitute > lastMoveThreshold)
                    {
                        System.Diagnostics.Debug.WriteLine("Magnitude: " + lastMove.Magnitute);
                        txtStatus.Text = "Unlocked with " + lastTouchPoints + " fingers UP";
                    }
                    e.Complete();
                }
            }
        }
        private void gestureRecognizer_ManipulationCompleted(object sender, ManipulationCompletedRoutedEventArgs e)
        {         
            // Don't forget to clear the list.
            //lastTouchPoints = pointerList.Count;
            this.pointerList.Clear();
            txtNumTouches.Text = "Touch Points: " + pointerList.Count + " /// " + lastTouchPoints;           
        }

        // Make sure you clear your list in whatever events make sense.
        private void Canvas_PointerReleased(object sender, PointerRoutedEventArgs e)
        {
            //lastTouchPoints = pointerList.Count;
            this.pointerList.Clear();
            txtNumTouches.Text = "Touch Points: " + pointerList.Count + " /// " + lastTouchPoints;
        }

        private void Canvas_PointerCanceled(object sender, PointerRoutedEventArgs e)
        {
            //lastTouchPoints = pointerList.Count;
            this.pointerList.Clear();
            txtNumTouches.Text = "Touch Points: " + pointerList.Count + " /// " + lastTouchPoints;
        }

        private void Canvas_PointerCaptureLost(object sender, PointerRoutedEventArgs e)
        {
            //lastTouchPoints = pointerList.Count;
            this.pointerList.Clear();
            txtNumTouches.Text = "Touch Points: " + pointerList.Count + " /// " + lastTouchPoints;
        }

        private void Canvas_PointerExited(object sender, PointerRoutedEventArgs e)
        {
            //lastTouchPoints = pointerList.Count;
            this.pointerList.Clear();
            txtNumTouches.Text = "Touch Points: " + pointerList.Count + " /// " + lastTouchPoints;
        }

        private struct Point3D
        {
            public double X { get; set; }
            public double Y { get; set; }
            public double Z { get; set; }
        }

        private struct LastSwipe
        {
            public DateTime Time { get; set; }
            public double Angle { get; set; }
            public int NumTouches {get; set;}
        }

        private struct LastMove
        {
            public DateTime Time { get; set; }
            public double Angle { get; set; }
            public double Magnitute { get; set; }
        }

        private LastSwipe calculateSwipe() { return new LastSwipe(); }
        private LastMove calculateMove(DateTime now, double accelX, double accelY)
        {
            LastMove returnObject = new LastMove();
            if (accelX == 0)
            {
                if (accelY > 0)
                {
                    returnObject.Angle = 90;
                }
                else
                {
                    returnObject.Angle = 270;
                }
            }
            else
            {
                returnObject.Angle = Math.Atan2(accelY, accelX);
            }
            returnObject.Time = now;
            returnObject.Magnitute = Math.Sqrt((accelX * accelX) + (accelY * accelY));
            return returnObject;
        }

        public void GestureInputProcessor(Windows.UI.Input.GestureRecognizer gr, Windows.UI.Xaml.UIElement target)
        {
            this.gestureRecognizer = gr;

            //Targeted Ui element to be performing gestures on it.  
            this.element = target;
            this.element.ManipulationMode = ManipulationModes.All;

            //Enable gesture settings for Tap,Hold,RightTap,CrossSlide  
            //this.gestureRecognizer.GestureSettings = Windows.UI.Input.GestureSettings.Tap | Windows.UI.Input.GestureSettings.Hold | Windows.UI.Input.GestureSettings.RightTap | Windows.UI.Input.GestureSettings.CrossSlide;
            //this.gestureRecognizer.GestureSettings = GestureSettings.ManipulationTranslateX | GestureSettings.ManipulationTranslateY;
            this.gestureRecognizer.GestureSettings = Windows.UI.Input.GestureSettings.ManipulationRotate | Windows.UI.Input.GestureSettings.ManipulationTranslateX | Windows.UI.Input.GestureSettings.ManipulationTranslateY |
            Windows.UI.Input.GestureSettings.ManipulationScale | Windows.UI.Input.GestureSettings.ManipulationRotateInertia | Windows.UI.Input.GestureSettings.ManipulationScaleInertia |
            Windows.UI.Input.GestureSettings.ManipulationTranslateInertia | Windows.UI.Input.GestureSettings.Tap; 

            // Set up pointer event handlers. These receive input events that are used by the gesture recognizer.  
            this.element.PointerCanceled += Canvas_PointerCanceled;
            this.element.PointerPressed += Canvas_PointerPressed;
            this.element.PointerReleased += Canvas_PointerReleased;
            this.element.PointerCaptureLost += Canvas_PointerCaptureLost;
            this.element.PointerExited += Canvas_PointerExited;
            this.element.ManipulationCompleted += gestureRecognizer_ManipulationCompleted;
            this.element.ManipulationDelta += gestureRecognizer_ManipulationDelta;
            this.element.ManipulationStarted += gestureRecognizer_ManipulationStarted;
            
            //this.element.PointerMoved += OnPointerMoved;

            // Set up event handlers to respond to gesture recognizer output  
            //gestureRecognizer.Holding += gestureRecognizer_Holding;
            //gestureRecognizer.Tapped += gestureRecognizer_Tapped;
            //gestureRecognizer.RightTapped += gestureRecognizer_RightTapped;

            //CrossSliding distance thresholds are disabled by default. Use CrossSlideThresholds to set these values.  
            Windows.UI.Input.CrossSlideThresholds cst = new Windows.UI.Input.CrossSlideThresholds();
            cst.SelectionStart = 2;
            cst.SpeedBumpStart = 3;
            cst.SpeedBumpEnd = 4;
            cst.RearrangeStart = 5;
            gestureRecognizer.CrossSlideHorizontally = true;//Enable horinzontal slide  
            gestureRecognizer.CrossSlideThresholds = cst;

            //gestureRecognizer.CrossSliding += gestureRecognizer_CrossSliding;

        }
    }
}