﻿using System.Collections.Generic;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Input;

public class NumberOfTouchPointsDetector
{
    #region Fields
    readonly HashSet<uint> _pointers = new HashSet<uint>();
    readonly UIElement _uiElement;
    #endregion

    public int NumberOfTouchPoints { get { return _pointers.Count; } }

    public NumberOfTouchPointsDetector(UIElement uiElement)
    {
        _uiElement = uiElement;
        HookupEvents();
    }

    public void UnHookEvents()
    {
        _uiElement.PointerPressed -= PointerPressed;
        _uiElement.PointerReleased -= PointerReleased;
        _uiElement.PointerExited -= PointerExited;
        _uiElement.PointerEntered -= PointerEntered;
        _uiElement.PointerCaptureLost -= PointerCaptureLost;
        _uiElement.PointerCanceled -= PointerCanceled;
    }

    #region private
    private void PointerPressed(object sender, PointerRoutedEventArgs e)
    {
        _pointers.Add(e.Pointer.PointerId);
    }

    private void PointerReleased(object sender, PointerRoutedEventArgs e)
    {
        _pointers.Remove(GetPoint(e));
    }

    private void PointerExited(object sender, PointerRoutedEventArgs e)
    {
        _pointers.Remove(GetPoint(e));
    }

    private void PointerEntered(object sender, PointerRoutedEventArgs e)
    {
        _pointers.Add(GetPoint(e));
    }

    private void PointerCaptureLost(object sender, PointerRoutedEventArgs e)
    {
        _pointers.Remove(GetPoint(e));
    }

    private void PointerCanceled(object sender, PointerRoutedEventArgs e)
    {
        _pointers.Clear();
    }

    uint GetPoint(PointerRoutedEventArgs e)
    {
        return e.GetCurrentPoint(_uiElement).PointerId;
    }

    private void HookupEvents()
    {
        _uiElement.PointerPressed += PointerPressed;
        _uiElement.PointerReleased += PointerReleased;
        _uiElement.PointerExited += PointerExited;
        _uiElement.PointerEntered += PointerEntered;
        _uiElement.PointerCaptureLost += PointerCaptureLost;
        _uiElement.PointerCanceled += PointerCanceled;
    }
    #endregion
}